"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.SocketTester = void 0;
var SubscriptionHandler = /** @class */ (function () {
    function SubscriptionHandler(resolver, predicate) {
        this.resolver = resolver;
        this.predicate = predicate;
    }
    SubscriptionHandler.prototype.test = function () {
        return this.predicate();
    };
    SubscriptionHandler.prototype.resolve = function () {
        this.resolver(null);
    };
    return SubscriptionHandler;
}());
var SocketSubscription = /** @class */ (function () {
    function SocketSubscription(socket, name) {
        var _this = this;
        this.responses = [];
        this.handlers = [];
        socket.on(name, function (data) {
            _this.onResponse(data);
        });
    }
    SocketSubscription.prototype.onResponse = function (data) {
        this.responses.push(data);
        this.handlers = this.handlers.filter(function (h) {
            if (h.test()) {
                h.resolve();
                return false;
            }
            return true;
        });
    };
    SocketSubscription.prototype.waitFor = function (predicate) {
        var _this = this;
        return new Promise(function (resolve, reject) {
            _this.handlers.push(new SubscriptionHandler(resolve, predicate));
        });
    };
    SocketSubscription.prototype.waitUntil = function (predicate) {
        var _this = this;
        return this.waitFor(function () {
            var last = _this.responses[_this.responses.length - 1];
            return last != null && predicate(last);
        });
    };
    SocketSubscription.prototype.waitForAny = function () {
        return this.waitForEvents(1);
    };
    SocketSubscription.prototype.waitForEvents = function (num) {
        var _this = this;
        return this.waitFor(function () { return _this.responses.length >= num; });
    };
    SocketSubscription.prototype.get = function (index) {
        return this.responses[index];
    };
    return SocketSubscription;
}());
var SocketTester = /** @class */ (function () {
    function SocketTester(socket) {
        this.socket = socket;
    }
    SocketTester.prototype.on = function (name) {
        return new SocketSubscription(this.socket, name);
    };
    return SocketTester;
}());
exports.SocketTester = SocketTester;
